package GCG.Clients.Extended;

import javax.swing.*;
import javax.swing.event.MouseInputAdapter;

import GCG.coloredVertex.ColoredVertex;
import GCG.coloredVertex.ColoredVertexGraphConstructor;
import GCG.protokoll.PROTOKOLL;
import abiturklassen.graph.Edge;
import abiturklassen.graph.Graph;
import abiturklassen.graph.Vertex;
import abiturklassen.linear.List;

import java.awt.*;
import java.awt.event.MouseEvent;

/**
 * 
 * @version 5.0 vom 20.12.2015
 * @author Klaus Bovermann
 *
 */
@SuppressWarnings("serial")
public class ExtendedGraphClientPane extends JPanel {

	private static int MAX_FARBEN = 6;

	private String graphString;
	private ColoredVertexGraphConstructor myBuilder;
	private Graph derGraph;

	private JButton[] vertexButtons;
	private int anzahlFarben;
	private Color[] farben;
	private String[] farbnamen;
	private String[] iconNamen;

	private DragListener drag;

	private ExtendedGraphClient meinClient;

	public ExtendedGraphClientPane(ExtendedGraphClient derClient) {
		super();
		this.meinClient = derClient;
		this.setLayout(null);// NO LAYOUT!!! -> ermöglicht freies Positionieren!
		myBuilder = new ColoredVertexGraphConstructor();
		derGraph = null;
		this.anzahlFarben = ExtendedGraphClientPane.MAX_FARBEN;
		farben = new Color[anzahlFarben];
		farbnamen = new String[anzahlFarben];
		iconNamen = new String[anzahlFarben];

		// Spielfarben...
		farben[0] = Color.BLUE;
		farbnamen[0] = "Blue";
		iconNamen[0] = "blau.jpg";
		farben[1] = Color.RED;
		farbnamen[1] = "Red";
		iconNamen[1] = "rot.jpg";
		farben[2] = Color.GREEN;
		farbnamen[2] = "Green";
		iconNamen[2] = "gruen.jpg";
		farben[3] = Color.PINK;
		farbnamen[3] = "Pink";
		iconNamen[3] = "pink.jpg";
		farben[4] = Color.CYAN;
		farbnamen[4] = "Cyan";
		iconNamen[4] = "cyan.jpg";
		farben[5] = Color.ORANGE;
		farbnamen[5] = "Orange";
		iconNamen[5] = "orange.jpg";

		drag = new DragListener();
	}

	/**
	 * @return the anzahlFarben
	 */
	public int getAnzahlFarben() {
		return anzahlFarben;
	}

	/**
	 * @param pAnzahlFarben
	 *            the anzahlFarben to set
	 */
	public void setAnzahlFarben(int pAnzahlFarben) {
		if (pAnzahlFarben <= ExtendedGraphClientPane.MAX_FARBEN && pAnzahlFarben > 0) {
			this.anzahlFarben = pAnzahlFarben;
		} else {
			this.anzahlFarben = ExtendedGraphClientPane.MAX_FARBEN;
		}
	}

	/**
	 * @param pGraphString
	 */
	public void GraphAnzeigen(String pGraphString) {
		if (pGraphString == null || pGraphString.equals("")) {
			// NIX ?? Wie kann das sein???
		} else {
			this.graphString = pGraphString;
			this.repaint();
		}
	}

	/**
	 * @param pKnotenName
	 * @param pFarbName
	 */
	public void updateColor(String pKnotenName, String pFarbName) {

		int color = Integer.parseInt(pFarbName);

		int numVertices = 0;

		List<Vertex> vertexList = derGraph.getVertices();

		vertexList.toFirst();

		while (vertexList.hasAccess()) {
			Vertex aVertex = vertexList.getContent();

			if (aVertex.getID().equals(pKnotenName)) {
				ColoredVertex cv = (ColoredVertex) aVertex;
				cv.setColor(color);

				vertexButtons[numVertices].setForeground(farben[color]);
				vertexButtons[numVertices].setOpaque(true);
				vertexButtons[numVertices].setBackground(farben[color]);
				vertexButtons[numVertices].setIcon(new ImageIcon(this.iconNamen[color]));

			}
			numVertices++;
			vertexList.next();
		}
	}

	public void paint(Graphics g) {
		super.paint(g);
		if (this.graphString == null || this.graphString.equals("")) {
			// NIX !!!
		} else {
			if (derGraph == null || derGraph.isEmpty()) {
				prepare();
				drawButtons();
			}
			drawEdges(g);
		}
	}

	private JButton getVertexByID(String id) {
		for (JButton b : vertexButtons) {
			if (b.getText().equals(id)) {
				return b;
			}
		}
		return null;
	}

	private void drawEdges(Graphics g) {
		abiturklassen.linear.List<Edge> edges = derGraph.getEdges();

		edges.toFirst();
		while (edges.hasAccess()) {
			Edge anEdge = edges.getContent();

			Vertex[] verts = anEdge.getVertices();

			JButton from = getVertexByID(verts[0].getID());
			JButton to = getVertexByID(verts[1].getID());

			g.drawLine(from.getBounds().x, from.getBounds().y, to.getBounds().x, to.getBounds().y);

			edges.next();
		}
	}

	private void prepare() {
		myBuilder.stringToGraph(this.graphString);

		derGraph = myBuilder.getGraph();

		int numVertices = 0;

		abiturklassen.linear.List<Vertex> vertexList = derGraph.getVertices();

		vertexList.toFirst();

		while (vertexList.hasAccess()) {
			numVertices++;
			vertexList.next();
		}

		// Schaffe Platz für hinreichend viele Buttons:
		vertexButtons = new JButton[numVertices];

		// Erzeuge hinrechend viele Buttons:
		vertexList.toFirst();

		double angle = 2.0 * Math.PI / ((double) numVertices);
		double radius = 150.0;
		double centerX = 320.0;
		double centerY = 200.0;
		numVertices = 0;
		while (vertexList.hasAccess()) {
			Vertex v = vertexList.getContent();

			JButton neuer;
			neuer = new JButton(v.getID());
			neuer.setName(v.getID());
			neuer.setOpaque(true);

			neuer.setBounds((int) (centerX + radius * Math.sin(angle * numVertices)),
					(int) (centerY + radius * Math.cos(angle * numVertices)), 25, 25);

			neuer.addMouseListener(drag);
			neuer.addMouseMotionListener(drag);

			vertexButtons[numVertices++] = neuer;
			this.add(neuer);
			vertexList.next();
		}
	}

	private void drawButtons() {
		abiturklassen.linear.List<Vertex> vertexList = derGraph.getVertices();
		vertexList.toFirst();

		vertexList.toFirst();
		int numVerteces = 0;
		while (vertexList.hasAccess()) {
			Vertex v = vertexList.getContent();
			ColoredVertex cv = (ColoredVertex) v;

			JButton dieser = vertexButtons[numVerteces];
			if (cv.isColored()) {
				int farbnummer = cv.getColor();
				dieser.setForeground(farben[farbnummer]);
				dieser.setBackground(farben[farbnummer]);
			} else {
				dieser.setForeground(Color.GRAY);
			}
			vertexList.next();
			numVerteces++;
		}
	}

	private void askForColoring(String knotenName) {
		String[] namen;
		namen = new String[this.anzahlFarben];
		for (int i = 0; i < this.anzahlFarben; i++) {
			namen[i] = this.farbnamen[i];
		}

		int i = JOptionPane.showOptionDialog(this, "Farbe fuer Knoten " + knotenName + "?", "Alternativen",
				JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, namen, namen[0]);

		String nachricht = PROTOKOLL.CS_FAERBE + PROTOKOLL.TRENNER + knotenName + PROTOKOLL.TRENNER + i;

		// diesen Knoten faerben!!
		this.meinClient.send(nachricht);
	}

	private class DragListener extends MouseInputAdapter {
		Point location;
		MouseEvent pressed;

		public void mousePressed(MouseEvent me) {
			pressed = me;
		}

		public void mouseClicked(MouseEvent me) {
			pressed = me;
			Component component = me.getComponent();
			String knotenName = component.getName();
			askForColoring(knotenName);
			repaint();
		}

		public void mouseDragged(MouseEvent me) {
			Component component = me.getComponent();
			location = component.getLocation(location);
			int x = location.x - pressed.getX() + me.getX();
			int y = location.y - pressed.getY() + me.getY();
			component.setLocation(x, y);
			repaint();
		}
	}
}
