package GCG.server;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;
import javax.swing.SwingConstants;

/**
 * Startet einen Server.
 * 
 * <p>
 * Der Benutzer muss aussuchen:
 * </p>
 * <ul>
 * <li>auf welchem Port lauscht der Server?</li>
 * <li>in welcher Datei ist der Graph beschrieben?</li>
 * <li>wie viele Farben sind erlaubt (1 bis 6)</li>
 * </ul>
 * 
 * @version 4.1 vom 27.7.2016
 * @author Klaus Bovermann
 */
@SuppressWarnings("serial")
public class GCG_ServerStarter extends JFrame {

	private String dateiname;
	private File currDir;

	private JButton ok_button;
	private JButton fileChooseButton;

	private JLabel farbenLabel;
	private JLabel graphDateiLabel;
	private JLabel portLabel;
	private JLabel graphNameLabel;

	private JTextField portTextField;

	private JSlider anzahlFarbenRegler;

	// Constructor
	public GCG_ServerStarter() {
		this.dateiname = "???";
		this.currDir = new File(".");

		this.setTitle("Server starten");
		this.setSize(367, 305);

		// pane with null layout
		JPanel contentPane = new JPanel(null);
		contentPane.setPreferredSize(new Dimension(300, 200));
		contentPane.setBackground(new Color(192, 192, 192));

		anzahlFarbenRegler = new JSlider();

		anzahlFarbenRegler.setMinimum(1);
		anzahlFarbenRegler.setMaximum(6);
		anzahlFarbenRegler.setMajorTickSpacing(1);
		anzahlFarbenRegler.setMinorTickSpacing(1);
		anzahlFarbenRegler.createStandardLabels(1);
		anzahlFarbenRegler.setPaintTicks(true);
		anzahlFarbenRegler.setPaintLabels(true);
		anzahlFarbenRegler.setValue(2);
		anzahlFarbenRegler.setBounds(124, 78, 104, 92);

		ok_button = new JButton();
		ok_button.setBounds(204, 159, 90, 35);
		ok_button.setBackground(new Color(214, 217, 223));
		ok_button.setForeground(new Color(0, 0, 0));
		ok_button.setEnabled(true);
		ok_button.setFont(new Font("sansserif", 0, 12));
		ok_button.setText("OK!");
		ok_button.setVisible(true);

		ok_button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				okButtonAction(evt);
			}
		});

		farbenLabel = new JLabel();
		farbenLabel.setBounds(6, 114, 104, 21);
		farbenLabel.setBackground(new Color(214, 217, 223));
		farbenLabel.setForeground(new Color(0, 0, 0));
		farbenLabel.setEnabled(true);
		farbenLabel.setFont(new Font("sansserif", 0, 12));
		farbenLabel.setText("Anzahl Farben");
		farbenLabel.setVisible(true);

		graphDateiLabel = new JLabel();
		graphDateiLabel.setBounds(114, 12, 180, 21);
		graphDateiLabel.setBackground(new Color(214, 217, 223));
		graphDateiLabel.setForeground(new Color(0, 0, 0));
		graphDateiLabel.setEnabled(true);
		graphDateiLabel.setFont(new Font("sansserif", 0, 12));
		graphDateiLabel.setText("Graphbeschreibung in Datei:");
		graphDateiLabel.setVisible(true);

		portLabel = new JLabel();
		portLabel.setBounds(6, 12, 90, 21);
		portLabel.setBackground(new Color(214, 217, 223));
		portLabel.setForeground(new Color(0, 0, 0));
		portLabel.setEnabled(true);
		portLabel.setFont(new Font("sansserif", 0, 12));
		portLabel.setText("Port");
		portLabel.setVisible(true);

		portTextField = new JTextField();
		portTextField.setBounds(6, 39, 90, 35);
		portTextField.setBackground(Color.WHITE);
		portTextField.setForeground(new Color(0, 0, 0));
		portTextField.setEnabled(true);
		portTextField.setFont(new Font("sansserif", 0, 12));
		portTextField.setText("55555");
		portTextField.setVisible(true);

		graphNameLabel = new JLabel();
		graphNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		graphNameLabel.setBorder(new LineBorder(new Color(0, 0, 0)));
		graphNameLabel.setBounds(114, 42, 117, 29);
		graphNameLabel.setBackground(Color.WHITE);
		graphNameLabel.setForeground(new Color(0, 0, 0));
		graphNameLabel.setEnabled(true);
		graphNameLabel.setFont(new Font("sansserif", 0, 12));
		graphNameLabel.setText(dateiname);
		graphNameLabel.setVisible(true);

		fileChooseButton = new JButton();

		Icon icon = new ImageIcon("open.png");
		fileChooseButton.setIcon(icon);

		fileChooseButton.setBounds(243, 43, 51, 29);
		fileChooseButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				graphAuswaehlen();
			}
		});

		// adding components to contentPane panel
		contentPane.add(ok_button);
		contentPane.add(farbenLabel);
		contentPane.add(graphDateiLabel);
		contentPane.add(portLabel);
		contentPane.add(portTextField);
		contentPane.add(fileChooseButton);
		contentPane.add(graphNameLabel);
		contentPane.add(anzahlFarbenRegler);

		// adding panel to JFrame and setting of window position and close
		// operation
		getContentPane().add(contentPane, BorderLayout.SOUTH);

		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setLocationRelativeTo(null);
		this.pack();
		this.setVisible(true);
	}

	private void graphAuswaehlen() {
		JFileChooser chooser = new JFileChooser();
		chooser.setCurrentDirectory(this.currDir);

		int rueckgabeWert = chooser.showOpenDialog(this);

		/* Abfrage, ob auf "Öffnen" geklickt wurde */
		if (rueckgabeWert == JFileChooser.APPROVE_OPTION) {
			dateiname = chooser.getSelectedFile().getName();
			chooser.getSelectedFile().getPath();
			this.currDir = chooser.getSelectedFile().getAbsoluteFile();
			this.graphNameLabel.setText(dateiname);
		}
	}

	private void okButtonAction(ActionEvent evt) {

		String dateiname = this.graphNameLabel.getText();
		if (dateiname.equals("???")) {
			JOptionPane.showMessageDialog(this, "Sie müssen eine GraphBeschreibungsDatei waehlen", "Fehler",
					JOptionPane.WARNING_MESSAGE);

		} else {

			int anzahlFarben = this.anzahlFarbenRegler.getValue();

			int port;
			try {
				port = Integer.parseInt(this.portTextField.getText());
				new GCG_Server(port, dateiname, anzahlFarben);
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(this, "Sie müssen für den Port eine Zahl eingeben", "Fehler",
						JOptionPane.WARNING_MESSAGE);
			}
		}
	}

	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					GCG_ServerStarter frame = new GCG_ServerStarter();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
}