package GCG.oldClient;

import javax.swing.*;
import javax.swing.JFrame;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.TitledBorder;

import GCG.protokoll.PROTOKOLL;

/**
 * Ein Frame, der in einem JLabel Ausgaben machen kann. Der Client benutzt
 * spaeter diesen Frame, um die Antworten, die er vom Server bekommt,
 * anzuzeigen.
 * 
 * @version 3.1 vom 4.11.2015
 * @author Klaus Bovermann
 */
@SuppressWarnings("serial")
public class AusgabeFrame extends JFrame {
	// Anfang Attribute

	private JLabel nameLabel;
	private JLabel anzahlFarbenAnzeige;
	private JTextArea Inhalt;
	private JScrollPane sp;

	private GraphClient derGraphClient;

	/**
	 * Hier werden der Graph sowie die Knotenfaerbungen angezeigt.
	 */
	private GraphDisplay myDisplay;
	// Ende Attribute

	public AusgabeFrame(String title) {
		// Frame-Initialisierung
		super(title);

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				dispose();
			}
		});
		int frameWidth = 400;
		int frameHeight = 200;
		setSize(frameWidth, frameHeight);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (d.width - getSize().width) / 2;
		int y = (d.height - getSize().height) / 2;
		setLocation(x, y);
		Panel cp = new Panel(null);
		getContentPane().add(cp);
		// Anfang Komponenten
		nameLabel = new JLabel();
		nameLabel.setBorder(new TitledBorder(null, "", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		nameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		nameLabel.setBounds(10, 10, 300, 16);
		nameLabel.setText("");
		nameLabel.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));
		cp.add(nameLabel);

		Inhalt = new JTextArea();
		Inhalt.setBorder(new TitledBorder(null, "", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		Inhalt.setFont(new Font("MS Sans Serif", Font.PLAIN, 13));

		sp = new JScrollPane(Inhalt);
		sp.setBounds(10, 30, 300, 100);
		cp.add(sp);

		JLabel lblAnzahlFarben = new JLabel("Anzahl Farben:");
		lblAnzahlFarben.setBounds(10, 142, 110, 16);
		cp.add(lblAnzahlFarben);

		anzahlFarbenAnzeige = new JLabel("??");
		anzahlFarbenAnzeige.setBounds(143, 142, 61, 16);
		cp.add(anzahlFarbenAnzeige);
		// Ende Komponenten

		// my part..
		myDisplay = new GraphDisplay(this);

		setResizable(false);
		setVisible(true);
	}

	/**
	 * @param derGraphClient
	 *            the derGraphClient to set
	 */
	public void setDerGraphClient(GraphClient derGraphClient) {
		this.derGraphClient = derGraphClient;
	}

	public void send(String nachricht) {
		this.derGraphClient.send(nachricht);
	}

	private void inhaltHinzu(String was) {
		String bisher = this.Inhalt.getText();
		String neu = was + "\n" + bisher;
		this.Inhalt.setText(neu);
	}

	private void syntaxFehlerAnzeigen(String was) {
		inhaltHinzu("Fehlerhafte Syntax in der Servernachricht <" + was + ">");
	}

	public void ausgeben(String was) {
		// this.Inhalt.append("\n" + "Empfangen: <" + was + ">");
		// Falls hier eine Nachricht ankommt, die informiert ueber den Graphen
		// bzw. eine Farbinformation, wird diese Nachricht an das Graph-display
		// weitergeleitet
		String[] teile = was.split(PROTOKOLL.TRENNER);
		switch (teile[0]) {
		case PROTOKOLL.SC_GRAPH:
			if (teile.length == 2) {
				inhaltHinzu("Graph: " + teile[1]);
				myDisplay.anzeigen(teile[1]);
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_GEFAERBT:
			if (teile.length == 3) {
				inhaltHinzu("Gefaerbt wurde Knoten " + teile[1] + " mit Farbe " + teile[2]);
				myDisplay.updateColor(teile[1], teile[2]);
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_AKTIV:
			if (teile.length == 1) {
				inhaltHinzu("Sie sind am Zug");
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_ERROR:
			if (teile.length == 2) {
				inhaltHinzu("Fehler: " + teile[1]);
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_FARBZAHL:
			if (teile.length == 2) {
				this.anzahlFarbenAnzeige.setText("" + teile[1]);
				this.myDisplay.farbzahlNeu(teile[1]);
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_KNOTEN:
			if (teile.length == 2) {
				inhaltHinzu("Knoten: " + teile[1]);
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_NAME:
			if (teile.length == 2) {
				this.nameLabel.setText(teile[1]);
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_OK:
			if (teile.length == 2) {
				inhaltHinzu("OK: " + teile[1]);
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_START:
			if (teile.length == 1) {
				inhaltHinzu("Das Spiel beginnt");
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_UNENTSCHIEDEN:
			if (teile.length == 1) {
				inhaltHinzu("Das Spiel endete unentschieden");
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_GEWINN_ENDE:
			if (teile.length == 2) {
				inhaltHinzu("Das Spiel ist beendet. Gewonnen hat " + teile[1]);
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		case PROTOKOLL.SC_VERBINDUNG_BEENDET:
			if (teile.length == 1) {
				inhaltHinzu("Die Verbindung wird beendet");
			} else {
				syntaxFehlerAnzeigen(was);
			}
			break;
		default:
			this.Inhalt.append("\n" + "Unbekannte Nachricht vom Server: <" + was + ">");
		}
	}
}
