package GCG.oldClient;

import javax.swing.*;
import javax.swing.event.MouseInputAdapter;

import GCG.coloredVertex.ColoredVertex;
import GCG.coloredVertex.ColoredVertexGraphConstructor;
import GCG.protokoll.PROTOKOLL;
import abiturklassen.graph.Edge;
import abiturklassen.graph.Graph;
import abiturklassen.graph.Vertex;
import abiturklassen.linear.List;

import java.awt.*;
import java.awt.event.MouseEvent;

/**
 * 
 * @version 3.1 vom 4.11.2015
 * @author Klaus Bovermann
 *
 */
@SuppressWarnings("serial")
public class GraphPane extends JPanel {

	private String graphString = "";
	private ColoredVertexGraphConstructor myBuilder;
	private Graph derGraph;

	private JButton[] vertexButtons;
	private int anzahlFarben;
	private Color[] farben;
	private String[] farbnamen;

	private DragListener drag;

	private GraphDisplay meinDisplay;

	public GraphPane(GraphDisplay dasDisplay) {
		super();

		this.meinDisplay = dasDisplay;

		this.setLayout(null);// NO LAYOUT!!! -> ermöglicht freies Positionieren!
		myBuilder = new ColoredVertexGraphConstructor();
		derGraph = null;
		this.anzahlFarben = 4;
		farben = new Color[4];
		farbnamen = new String[4];

		// Spielfarben...
		farben[0] = Color.BLUE;
		farbnamen[0] = "Blue";
		farben[1] = Color.RED;
		farbnamen[1] = "Red";
		farben[2] = Color.GREEN;
		farbnamen[2] = "Green";
		farben[3] = Color.PINK;
		farbnamen[3] = "Pink";
		drag = new DragListener();
	}

	/**
	 * @return the anzahlFarben
	 */
	public int getAnzahlFarben() {
		return anzahlFarben;
	}

	/**
	 * @param anzahlFarben
	 *            the anzahlFarben to set
	 */
	public void setAnzahlFarben(int anzahlFarben) {
		if (anzahlFarben <= 4 && anzahlFarben > 0) {
			this.anzahlFarben = anzahlFarben;
		} else {
			this.anzahlFarben = 4;
		}
	}

	public void GraphAnzeigen(String graphString) {
		this.graphString = graphString;
		this.repaint();
	}

	public void updateColor(String knotenName, String farbName) {

		int color = Integer.parseInt(farbName);

		int numVertices = 0;

		List<Vertex> vertexList = derGraph.getVertices();

		vertexList.toFirst();

		while (vertexList.hasAccess()) {
			Vertex aVertex = vertexList.getContent();

			if (aVertex.getID().equals(knotenName)) {
				ColoredVertex cv = (ColoredVertex) aVertex;
				cv.setColor(color);

				// System.out.println("GEFUNDEN: " + aVertex.getID() + " ist Nr.
				// " + numVertices);

				vertexButtons[numVertices].setForeground(farben[color]);
				vertexButtons[numVertices].setBackground(farben[color]);
				vertexButtons[numVertices].setOpaque(true);

			}
			numVertices++;
			vertexList.next();
		}
	}

	public void paint(Graphics g) {
		super.paint(g);
		if (derGraph == null) {
			prepare();
			drawButtons();
		}

		drawEdges(g);

		// g.drawString(txt, 50, 100);
	}

	private JButton getVertexByID(String id) {
		for (JButton b : vertexButtons) {
			if (b.getText().equals(id)) {
				return b;
			}
		}

		return null;
	}

	private void drawEdges(Graphics g) {
		abiturklassen.linear.List<Edge> edges = derGraph.getEdges();

		edges.toFirst();

		while (edges.hasAccess()) {
			Edge anEdge = edges.getContent();

			Vertex[] verts = anEdge.getVertices();

			JButton from = getVertexByID(verts[0].getID());
			JButton to = getVertexByID(verts[1].getID());

			g.drawLine(from.getBounds().x, from.getBounds().y, to.getBounds().x, to.getBounds().y);

			edges.next();

		}

	}

	private void prepare() {
		myBuilder.stringToGraph(this.graphString);

		derGraph = myBuilder.getGraph();

		int numVerteces = 0;

		abiturklassen.linear.List<Vertex> vertexList = derGraph.getVertices();

		vertexList.toFirst();

		while (vertexList.hasAccess()) {
			numVerteces++;
			vertexList.next();
		}

		// Schaffe Platz für hinreichend viele Buttons:
		vertexButtons = new JButton[numVerteces];

		// Erzeuge hinrechend viele Buttons:
		vertexList.toFirst();

		double angle = 2.0 * Math.PI / ((double) numVerteces);
		double radius = 150.0;
		double centerX = 320.0;
		double centerY = 200.0;
		numVerteces = 0;
		while (vertexList.hasAccess()) {

			Vertex v = vertexList.getContent();
			JButton neuer = new JButton(v.getID());

			neuer.setName(v.getID());

			neuer.setBounds((int) (centerX + radius * Math.sin(angle * numVerteces)),
					(int) (centerY + radius * Math.cos(angle * numVerteces)), 25, 25);

			neuer.addMouseListener(drag);
			neuer.addMouseMotionListener(drag);

			vertexButtons[numVerteces++] = neuer;
			this.add(neuer);
			vertexList.next();
		}
	}

	private void drawButtons() {
		abiturklassen.linear.List<Vertex> vertexList = derGraph.getVertices();
		vertexList.toFirst();

		vertexList.toFirst();
		int numVerteces = 0;
		while (vertexList.hasAccess()) {
			Vertex v = vertexList.getContent();
			ColoredVertex cv = (ColoredVertex) v;

			JButton dieser = vertexButtons[numVerteces];
			if (cv.isColored()) {
				int farbnummer = cv.getColor();
				dieser.setForeground(farben[farbnummer]);
			} else {
				dieser.setForeground(Color.GRAY);
			}
			vertexList.next();
			numVerteces++;
		}

	}

	private void askForColoring(String knotenName) {
		// Object[] options = {1,2,3};
		String[] namen;
		namen = new String[this.anzahlFarben];
		for (int i = 0; i < this.anzahlFarben; i++) {
			namen[i] = this.farbnamen[i];
		}

		// JOptionPane.showInputDialog(this, "In welcher Farbe soll der Knoten
		// gefärbt werden?", "Farbauswahl",
		// JOptionPane.QUESTION_MESSAGE, null, namen, 0);

		int i = JOptionPane.showOptionDialog(this, "Farbe fuer Knoten " + knotenName + "?", "Alternativen",
				JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, namen, namen[0]);
		// Color dieseFarbe = farben[i];
		String nachricht = PROTOKOLL.CS_FAERBE + PROTOKOLL.TRENNER + knotenName + PROTOKOLL.TRENNER + i;

		// diesen Knoten faerben!!
		// System.out.println("Nachricht von Pane an Server: " + nachricht);
		this.meinDisplay.send(nachricht);

	}

	private class DragListener extends MouseInputAdapter {
		Point location;
		MouseEvent pressed;

		public void mousePressed(MouseEvent me) {
			// System.out.println("pressed: " + pressed);
			pressed = me;
		}

		public void mouseClicked(MouseEvent me) {
			// System.out.println("clicked: " + pressed);
			pressed = me;
			Component component = me.getComponent();
			String knotenName = component.getName();
			askForColoring(knotenName);
			repaint();
		}

		public void mouseDragged(MouseEvent me) {
			Component component = me.getComponent();
			location = component.getLocation(location);
			int x = location.x - pressed.getX() + me.getX();
			int y = location.y - pressed.getY() + me.getY();
			component.setLocation(x, y);
			repaint();

		}
	}
}
